package nc.bs.bd.pfxx.plugin;

import java.util.ArrayList;
import java.util.List;
import nc.bs.framework.common.NCLocator;
import nc.bs.pfxx.ISwapContext;
import nc.bs.pfxx.plugin.AbstractPfxxPlugin;
import nc.itf.bd.psn.psndoc.IPsndocService;
import nc.md.data.access.NCObject;
import nc.md.model.MetaDataException;
import nc.md.persist.framework.IMDPersistenceQueryService;
import nc.vo.bd.psn.PsndocVO;
import nc.vo.bd.psn.PsnjobVO;
import nc.vo.ml.NCLangRes4VoTransl;
import nc.vo.pfxx.auxiliary.AggxsysregisterVO;
import nc.vo.pfxx.util.PfxxPluginUtils;
import nc.vo.pub.BusinessException;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;

public class PsndocPfxxPlugin extends AbstractPfxxPlugin {
	private IPsndocService service = null;
	private IMDPersistenceQueryService queryService = null;

	protected Object processBill(Object vo, ISwapContext swapContext, AggxsysregisterVO aggxsysvo)
			throws BusinessException {
		PsndocVO resvo = (PsndocVO) vo;
		this.checkBeforeImport(resvo);
		String vopk = PfxxPluginUtils.queryBillPKBeforeSaveOrUpdate(swapContext.getBilltype(), swapContext.getDocID(),
				swapContext.getOrgPk());
		resvo = this.getPsndocVOForSave(resvo, vopk);
		if (StringUtils.isBlank(vopk)) {
			resvo = this.getPsndocService().insertPsndoc(resvo, false);
			PfxxPluginUtils.addDocIDVsPKContrast(swapContext.getBilltype(), swapContext.getDocID(),
					swapContext.getOrgPk(), resvo.getPrimaryKey());
		} else {
			this.getPsndocService().updatePsndoc(resvo, false);
		}

		return vopk;
	}

	private void checkBeforeImport(PsndocVO resvo) throws BusinessException {
		if (ArrayUtils.isEmpty(resvo.getPsnjobs())) {
			throw new BusinessException(NCLangRes4VoTransl.getNCLangRes().getStrByID("10140psn", "010140psn0059"));
		}
	}

	private PsndocVO getPsndocVOForSave(PsndocVO resvo, String vopk) throws MetaDataException, BusinessException {
		List<PsnjobVO> list = new ArrayList();
		this.addPsnjobVOToList(resvo, list, 2);
		if (StringUtils.isEmpty(vopk)) {
			resvo.setStatus(2);
		} else {
			PsndocVO oldvo = this.queryOldPSndovCO(vopk);
			resvo.setPrimaryKey(vopk);
			resvo.setStatus(1);
			resvo.setCreationtime(oldvo.getCreationtime());
			resvo.setCreator(oldvo.getCreator());
			this.addPsnjobVOToList(oldvo, list, 3);
		}

		resvo.setPsnjobs((PsnjobVO[]) list.toArray(new PsnjobVO[0]));
		return resvo;
	}

	private void addPsnjobVOToList(PsndocVO resvo, List<PsnjobVO> list, int status) {
		PsnjobVO[] jobvos = resvo.getPsnjobs();
		if (jobvos != null) {
			PsnjobVO[] arr$ = jobvos;
			int len$ = jobvos.length;

			for (int i$ = 0; i$ < len$; ++i$) {
				PsnjobVO jobvo = arr$[i$];
				jobvo.setStatus(status);
				list.add(jobvo);
			}
		}

	}

	private PsndocVO queryOldPSndovCO(String vopk) throws MetaDataException, BusinessException {
		NCObject obj = this.getIPsndocQueryService().queryBillOfNCObjectByPK(PsndocVO.class, vopk);
		if (obj == null) {
			throw new BusinessException(NCLangRes4VoTransl.getNCLangRes().getStrByID("10140psn", "110140psn0026"));
		} else {
			PsndocVO oldvo = (PsndocVO) obj.getContainmentObject();
			return oldvo;
		}
	}

	private IPsndocService getPsndocService() {
		if (this.service == null) {
			this.service = (IPsndocService) NCLocator.getInstance().lookup(IPsndocService.class);
		}

		return this.service;
	}

	private IMDPersistenceQueryService getIPsndocQueryService() {
		if (this.queryService == null) {
			this.queryService = (IMDPersistenceQueryService) NCLocator.getInstance()
					.lookup(IMDPersistenceQueryService.class);
		}

		return this.queryService;
	}
}